# Do not edit this file; it was automatically generated.

from nidaqmx.scale import _ScaleAlternateConstructor
from nidaqmx.task.channels._channel import Channel
from nidaqmx.constants import (
    AOIdleOutputBehavior, CurrentUnits, DataTransferActiveTransferMode,
    DigitalWidthUnits, FuncGenType, ModulationType,
    OutputDataTransferCondition, ResolutionType, SourceSelection,
    TerminalConfiguration, UsageTypeAO, VoltageUnits)


class AOChannel(Channel):
    """
    Represents one or more analog output virtual channels and their properties.
    """
    __slots__ = ()

    def __repr__(self):
        return f'AOChannel(name={self._name})'

    @property
    def ao_common_mode_offset(self):
        """
        float: Specifies the common-mode offset of the AO channel. Use
            the property only when Terminal Configuration is set to
            Differential.
        """

        val = self._interpreter.get_chan_attribute_double(self._handle, self._name, 0x31cc)
        return val

    @ao_common_mode_offset.setter
    def ao_common_mode_offset(self, val):
        self._interpreter.set_chan_attribute_double(self._handle, self._name, 0x31cc, val)

    @ao_common_mode_offset.deleter
    def ao_common_mode_offset(self):
        self._interpreter.reset_chan_attribute(self._handle, self._name, 0x31cc)

    @property
    def ao_current_units(self):
        """
        :class:`nidaqmx.constants.CurrentUnits`: Specifies in what units
            to generate current on the channel. Write data to the
            channel in the units you select.
        """

        val = self._interpreter.get_chan_attribute_int32(self._handle, self._name, 0x1109)
        return CurrentUnits(val)

    @ao_current_units.setter
    def ao_current_units(self, val):
        val = val.value
        self._interpreter.set_chan_attribute_int32(self._handle, self._name, 0x1109, val)

    @ao_current_units.deleter
    def ao_current_units(self):
        self._interpreter.reset_chan_attribute(self._handle, self._name, 0x1109)

    @property
    def ao_custom_scale(self):
        """
        :class:`nidaqmx.system.scale.Scale`: Specifies the name of a
            custom scale for the channel.
        """

        val = self._interpreter.get_chan_attribute_string(self._handle, self._name, 0x1188)
        return _ScaleAlternateConstructor(val, self._interpreter)

    @ao_custom_scale.setter
    def ao_custom_scale(self, val):
        val = val.name
        self._interpreter.set_chan_attribute_string(self._handle, self._name, 0x1188, val)

    @ao_custom_scale.deleter
    def ao_custom_scale(self):
        self._interpreter.reset_chan_attribute(self._handle, self._name, 0x1188)

    @property
    def ao_dac_offset_ext_src(self):
        """
        str: Specifies the source of the DAC offset voltage if
            **ao_dac_offset_src** is **SourceSelection.EXTERNAL**. The
            valid sources for this signal vary by device.
        """

        val = self._interpreter.get_chan_attribute_string(self._handle, self._name, 0x2254)
        return val

    @ao_dac_offset_ext_src.setter
    def ao_dac_offset_ext_src(self, val):
        self._interpreter.set_chan_attribute_string(self._handle, self._name, 0x2254, val)

    @ao_dac_offset_ext_src.deleter
    def ao_dac_offset_ext_src(self):
        self._interpreter.reset_chan_attribute(self._handle, self._name, 0x2254)

    @property
    def ao_dac_offset_src(self):
        """
        :class:`nidaqmx.constants.SourceSelection`: Specifies the source
            of the DAC offset voltage. The value of this voltage source
            determines the full-scale value of the DAC.
        """

        val = self._interpreter.get_chan_attribute_int32(self._handle, self._name, 0x2253)
        return SourceSelection(val)

    @ao_dac_offset_src.setter
    def ao_dac_offset_src(self, val):
        val = val.value
        self._interpreter.set_chan_attribute_int32(self._handle, self._name, 0x2253, val)

    @ao_dac_offset_src.deleter
    def ao_dac_offset_src(self):
        self._interpreter.reset_chan_attribute(self._handle, self._name, 0x2253)

    @property
    def ao_dac_offset_val(self):
        """
        float: Specifies in volts the value of the DAC offset voltage.
            To achieve best accuracy, the DAC offset value should be
            hand calibrated.
        """

        val = self._interpreter.get_chan_attribute_double(self._handle, self._name, 0x2255)
        return val

    @ao_dac_offset_val.setter
    def ao_dac_offset_val(self, val):
        self._interpreter.set_chan_attribute_double(self._handle, self._name, 0x2255, val)

    @ao_dac_offset_val.deleter
    def ao_dac_offset_val(self):
        self._interpreter.reset_chan_attribute(self._handle, self._name, 0x2255)

    @property
    def ao_dac_ref_allow_conn_to_gnd(self):
        """
        bool: Specifies whether to allow grounding the internal DAC
            reference at run time. You must set this property to True
            and set **ao_dac_ref_src** to **SourceSelection.INTERNAL**
            before you can set **ao_dac_ref_conn_to_gnd** to True.
        """

        val = self._interpreter.get_chan_attribute_bool(self._handle, self._name, 0x1830)
        return val

    @ao_dac_ref_allow_conn_to_gnd.setter
    def ao_dac_ref_allow_conn_to_gnd(self, val):
        self._interpreter.set_chan_attribute_bool(self._handle, self._name, 0x1830, val)

    @ao_dac_ref_allow_conn_to_gnd.deleter
    def ao_dac_ref_allow_conn_to_gnd(self):
        self._interpreter.reset_chan_attribute(self._handle, self._name, 0x1830)

    @property
    def ao_dac_ref_conn_to_gnd(self):
        """
        bool: Specifies whether to ground the internal DAC reference.
            Grounding the internal DAC reference has the effect of
            grounding all analog output channels and stopping waveform
            generation across all analog output channels regardless of
            whether the channels belong to the current task. You can
            ground the internal DAC reference only when
            **ao_dac_ref_src** is **SourceSelection.INTERNAL** and
            **ao_dac_ref_allow_conn_to_gnd** is True.
        """

        val = self._interpreter.get_chan_attribute_bool(self._handle, self._name, 0x130)
        return val

    @ao_dac_ref_conn_to_gnd.setter
    def ao_dac_ref_conn_to_gnd(self, val):
        self._interpreter.set_chan_attribute_bool(self._handle, self._name, 0x130, val)

    @ao_dac_ref_conn_to_gnd.deleter
    def ao_dac_ref_conn_to_gnd(self):
        self._interpreter.reset_chan_attribute(self._handle, self._name, 0x130)

    @property
    def ao_dac_ref_ext_src(self):
        """
        str: Specifies the source of the DAC reference voltage if
            **ao_dac_ref_src** is **SourceSelection.EXTERNAL**. The
            valid sources for this signal vary by device.
        """

        val = self._interpreter.get_chan_attribute_string(self._handle, self._name, 0x2252)
        return val

    @ao_dac_ref_ext_src.setter
    def ao_dac_ref_ext_src(self, val):
        self._interpreter.set_chan_attribute_string(self._handle, self._name, 0x2252, val)

    @ao_dac_ref_ext_src.deleter
    def ao_dac_ref_ext_src(self):
        self._interpreter.reset_chan_attribute(self._handle, self._name, 0x2252)

    @property
    def ao_dac_ref_src(self):
        """
        :class:`nidaqmx.constants.SourceSelection`: Specifies the source
            of the DAC reference voltage. The value of this voltage
            source determines the full-scale value of the DAC.
        """

        val = self._interpreter.get_chan_attribute_int32(self._handle, self._name, 0x132)
        return SourceSelection(val)

    @ao_dac_ref_src.setter
    def ao_dac_ref_src(self, val):
        val = val.value
        self._interpreter.set_chan_attribute_int32(self._handle, self._name, 0x132, val)

    @ao_dac_ref_src.deleter
    def ao_dac_ref_src(self):
        self._interpreter.reset_chan_attribute(self._handle, self._name, 0x132)

    @property
    def ao_dac_ref_val(self):
        """
        float: Specifies in volts the value of the DAC reference
            voltage. This voltage determines the full-scale range of the
            DAC. Smaller reference voltages result in smaller ranges,
            but increased resolution.
        """

        val = self._interpreter.get_chan_attribute_double(self._handle, self._name, 0x1832)
        return val

    @ao_dac_ref_val.setter
    def ao_dac_ref_val(self, val):
        self._interpreter.set_chan_attribute_double(self._handle, self._name, 0x1832, val)

    @ao_dac_ref_val.deleter
    def ao_dac_ref_val(self):
        self._interpreter.reset_chan_attribute(self._handle, self._name, 0x1832)

    @property
    def ao_dac_rng_high(self):
        """
        float: Specifies the upper limit of the output range of the
            device. This value is in the native units of the device. On
            E Series devices, for example, the native units is volts.
        """

        val = self._interpreter.get_chan_attribute_double(self._handle, self._name, 0x182e)
        return val

    @ao_dac_rng_high.setter
    def ao_dac_rng_high(self, val):
        self._interpreter.set_chan_attribute_double(self._handle, self._name, 0x182e, val)

    @ao_dac_rng_high.deleter
    def ao_dac_rng_high(self):
        self._interpreter.reset_chan_attribute(self._handle, self._name, 0x182e)

    @property
    def ao_dac_rng_low(self):
        """
        float: Specifies the lower limit of the output range of the
            device. This value is in the native units of the device. On
            E Series devices, for example, the native units is volts.
        """

        val = self._interpreter.get_chan_attribute_double(self._handle, self._name, 0x182d)
        return val

    @ao_dac_rng_low.setter
    def ao_dac_rng_low(self, val):
        self._interpreter.set_chan_attribute_double(self._handle, self._name, 0x182d, val)

    @ao_dac_rng_low.deleter
    def ao_dac_rng_low(self):
        self._interpreter.reset_chan_attribute(self._handle, self._name, 0x182d)

    @property
    def ao_data_xfer_mech(self):
        """
        :class:`nidaqmx.constants.DataTransferActiveTransferMode`:
            Specifies the data transfer mode for the device.
        """

        val = self._interpreter.get_chan_attribute_int32(self._handle, self._name, 0x134)
        return DataTransferActiveTransferMode(val)

    @ao_data_xfer_mech.setter
    def ao_data_xfer_mech(self, val):
        val = val.value
        self._interpreter.set_chan_attribute_int32(self._handle, self._name, 0x134, val)

    @ao_data_xfer_mech.deleter
    def ao_data_xfer_mech(self):
        self._interpreter.reset_chan_attribute(self._handle, self._name, 0x134)

    @property
    def ao_data_xfer_req_cond(self):
        """
        :class:`nidaqmx.constants.OutputDataTransferCondition`:
            Specifies under what condition to transfer data from the
            buffer to the onboard memory of the device.
        """

        val = self._interpreter.get_chan_attribute_int32(self._handle, self._name, 0x183c)
        return OutputDataTransferCondition(val)

    @ao_data_xfer_req_cond.setter
    def ao_data_xfer_req_cond(self, val):
        val = val.value
        self._interpreter.set_chan_attribute_int32(self._handle, self._name, 0x183c, val)

    @ao_data_xfer_req_cond.deleter
    def ao_data_xfer_req_cond(self):
        self._interpreter.reset_chan_attribute(self._handle, self._name, 0x183c)

    @property
    def ao_dev_scaling_coeff(self):
        """
        List[float]: Indicates the coefficients of a linear equation
            that NI-DAQmx uses to scale values from a voltage to the
            native format of the device. Each element of the list
            corresponds to a term of the equation. The first element of
            the list corresponds to the y-intercept, and the second
            element corresponds to the slope. Scaling coefficients do
            not account for any custom scales that may be applied to the
            channel.
        """

        val = self._interpreter.get_chan_attribute_double_array(self._handle, self._name, 0x1931)
        return val

    @property
    def ao_enhanced_image_rejection_enable(self):
        """
        bool: Specifies whether to enable the DAC interpolation filter.
            Disable the interpolation filter to improve DAC signal-to-
            noise ratio at the expense of degraded image rejection.
        """

        val = self._interpreter.get_chan_attribute_bool(self._handle, self._name, 0x2241)
        return val

    @ao_enhanced_image_rejection_enable.setter
    def ao_enhanced_image_rejection_enable(self, val):
        self._interpreter.set_chan_attribute_bool(self._handle, self._name, 0x2241, val)

    @ao_enhanced_image_rejection_enable.deleter
    def ao_enhanced_image_rejection_enable(self):
        self._interpreter.reset_chan_attribute(self._handle, self._name, 0x2241)

    @property
    def ao_filter_delay(self):
        """
        float: Specifies the amount of time between when the sample is
            written by the host device and when the sample is output by
            the DAC. This value is in the units you specify with
            **ao_filter_delay_units**.
        """

        val = self._interpreter.get_chan_attribute_double(self._handle, self._name, 0x3075)
        return val

    @ao_filter_delay.setter
    def ao_filter_delay(self, val):
        self._interpreter.set_chan_attribute_double(self._handle, self._name, 0x3075, val)

    @ao_filter_delay.deleter
    def ao_filter_delay(self):
        self._interpreter.reset_chan_attribute(self._handle, self._name, 0x3075)

    @property
    def ao_filter_delay_adjustment(self):
        """
        float: Specifies an additional amount of time to wait between
            when the sample is written by the host device and when the
            sample is output by the DAC. This delay adjustment is in
            addition to the value indicated by **ao_filter_delay**. This
            delay adjustment is in the units you specify with
            **ao_filter_delay_units**.
        """

        val = self._interpreter.get_chan_attribute_double(self._handle, self._name, 0x3072)
        return val

    @ao_filter_delay_adjustment.setter
    def ao_filter_delay_adjustment(self, val):
        self._interpreter.set_chan_attribute_double(self._handle, self._name, 0x3072, val)

    @ao_filter_delay_adjustment.deleter
    def ao_filter_delay_adjustment(self):
        self._interpreter.reset_chan_attribute(self._handle, self._name, 0x3072)

    @property
    def ao_filter_delay_units(self):
        """
        :class:`nidaqmx.constants.DigitalWidthUnits`: Specifies the
            units of **ao_filter_delay** and
            **ao_filter_delay_adjustment**.
        """

        val = self._interpreter.get_chan_attribute_int32(self._handle, self._name, 0x3076)
        return DigitalWidthUnits(val)

    @ao_filter_delay_units.setter
    def ao_filter_delay_units(self, val):
        val = val.value
        self._interpreter.set_chan_attribute_int32(self._handle, self._name, 0x3076, val)

    @ao_filter_delay_units.deleter
    def ao_filter_delay_units(self):
        self._interpreter.reset_chan_attribute(self._handle, self._name, 0x3076)

    @property
    def ao_func_gen_amplitude(self):
        """
        float: Specifies the zero-to-peak amplitude of the waveform to
            generate in volts. Zero and negative values are valid.
        """

        val = self._interpreter.get_chan_attribute_double(self._handle, self._name, 0x2a1a)
        return val

    @ao_func_gen_amplitude.setter
    def ao_func_gen_amplitude(self, val):
        self._interpreter.set_chan_attribute_double(self._handle, self._name, 0x2a1a, val)

    @ao_func_gen_amplitude.deleter
    def ao_func_gen_amplitude(self):
        self._interpreter.reset_chan_attribute(self._handle, self._name, 0x2a1a)

    @property
    def ao_func_gen_fm_deviation(self):
        """
        float: Specifies the FM deviation in hertz per volt when
            **ao_func_gen_modulation_type** is **ModulationType.FM**.
        """

        val = self._interpreter.get_chan_attribute_double(self._handle, self._name, 0x2a23)
        return val

    @ao_func_gen_fm_deviation.setter
    def ao_func_gen_fm_deviation(self, val):
        self._interpreter.set_chan_attribute_double(self._handle, self._name, 0x2a23, val)

    @ao_func_gen_fm_deviation.deleter
    def ao_func_gen_fm_deviation(self):
        self._interpreter.reset_chan_attribute(self._handle, self._name, 0x2a23)

    @property
    def ao_func_gen_freq(self):
        """
        float: Specifies the frequency of the waveform to generate in
            hertz.
        """

        val = self._interpreter.get_chan_attribute_double(self._handle, self._name, 0x2a19)
        return val

    @ao_func_gen_freq.setter
    def ao_func_gen_freq(self, val):
        self._interpreter.set_chan_attribute_double(self._handle, self._name, 0x2a19, val)

    @ao_func_gen_freq.deleter
    def ao_func_gen_freq(self):
        self._interpreter.reset_chan_attribute(self._handle, self._name, 0x2a19)

    @property
    def ao_func_gen_modulation_type(self):
        """
        :class:`nidaqmx.constants.ModulationType`: Specifies if the
            device generates a modulated version of the waveform using
            the original waveform as a carrier and input from an
            external terminal as the signal.
        """

        val = self._interpreter.get_chan_attribute_int32(self._handle, self._name, 0x2a22)
        return ModulationType(val)

    @ao_func_gen_modulation_type.setter
    def ao_func_gen_modulation_type(self, val):
        val = val.value
        self._interpreter.set_chan_attribute_int32(self._handle, self._name, 0x2a22, val)

    @ao_func_gen_modulation_type.deleter
    def ao_func_gen_modulation_type(self):
        self._interpreter.reset_chan_attribute(self._handle, self._name, 0x2a22)

    @property
    def ao_func_gen_offset(self):
        """
        float: Specifies the voltage offset of the waveform to generate.
        """

        val = self._interpreter.get_chan_attribute_double(self._handle, self._name, 0x2a1b)
        return val

    @ao_func_gen_offset.setter
    def ao_func_gen_offset(self, val):
        self._interpreter.set_chan_attribute_double(self._handle, self._name, 0x2a1b, val)

    @ao_func_gen_offset.deleter
    def ao_func_gen_offset(self):
        self._interpreter.reset_chan_attribute(self._handle, self._name, 0x2a1b)

    @property
    def ao_func_gen_square_duty_cycle(self):
        """
        float: Specifies the square wave duty cycle of the waveform to
            generate.
        """

        val = self._interpreter.get_chan_attribute_double(self._handle, self._name, 0x2a1c)
        return val

    @ao_func_gen_square_duty_cycle.setter
    def ao_func_gen_square_duty_cycle(self, val):
        self._interpreter.set_chan_attribute_double(self._handle, self._name, 0x2a1c, val)

    @ao_func_gen_square_duty_cycle.deleter
    def ao_func_gen_square_duty_cycle(self):
        self._interpreter.reset_chan_attribute(self._handle, self._name, 0x2a1c)

    @property
    def ao_func_gen_start_phase(self):
        """
        float: Specifies the starting phase in degrees of the waveform
            to generate.
        """

        val = self._interpreter.get_chan_attribute_double(self._handle, self._name, 0x31c4)
        return val

    @ao_func_gen_start_phase.setter
    def ao_func_gen_start_phase(self, val):
        self._interpreter.set_chan_attribute_double(self._handle, self._name, 0x31c4, val)

    @ao_func_gen_start_phase.deleter
    def ao_func_gen_start_phase(self):
        self._interpreter.reset_chan_attribute(self._handle, self._name, 0x31c4)

    @property
    def ao_func_gen_type(self):
        """
        :class:`nidaqmx.constants.FuncGenType`: Specifies the kind of
            the waveform to generate.
        """

        val = self._interpreter.get_chan_attribute_int32(self._handle, self._name, 0x2a18)
        return FuncGenType(val)

    @ao_func_gen_type.setter
    def ao_func_gen_type(self, val):
        val = val.value
        self._interpreter.set_chan_attribute_int32(self._handle, self._name, 0x2a18, val)

    @ao_func_gen_type.deleter
    def ao_func_gen_type(self):
        self._interpreter.reset_chan_attribute(self._handle, self._name, 0x2a18)

    @property
    def ao_gain(self):
        """
        float: Specifies in decibels the gain factor to apply to the
            channel.
        """

        val = self._interpreter.get_chan_attribute_double(self._handle, self._name, 0x118)
        return val

    @ao_gain.setter
    def ao_gain(self, val):
        self._interpreter.set_chan_attribute_double(self._handle, self._name, 0x118, val)

    @ao_gain.deleter
    def ao_gain(self):
        self._interpreter.reset_chan_attribute(self._handle, self._name, 0x118)

    @property
    def ao_idle_output_behavior(self):
        """
        :class:`nidaqmx.constants.AOIdleOutputBehavior`: Specifies the
            state of the channel when no generation is in progress.
        """

        val = self._interpreter.get_chan_attribute_int32(self._handle, self._name, 0x2240)
        return AOIdleOutputBehavior(val)

    @ao_idle_output_behavior.setter
    def ao_idle_output_behavior(self, val):
        val = val.value
        self._interpreter.set_chan_attribute_int32(self._handle, self._name, 0x2240, val)

    @ao_idle_output_behavior.deleter
    def ao_idle_output_behavior(self):
        self._interpreter.reset_chan_attribute(self._handle, self._name, 0x2240)

    @property
    def ao_load_impedance(self):
        """
        float: Specifies in ohms the load impedance connected to the
            analog output channel.
        """

        val = self._interpreter.get_chan_attribute_double(self._handle, self._name, 0x121)
        return val

    @ao_load_impedance.setter
    def ao_load_impedance(self, val):
        self._interpreter.set_chan_attribute_double(self._handle, self._name, 0x121, val)

    @ao_load_impedance.deleter
    def ao_load_impedance(self):
        self._interpreter.reset_chan_attribute(self._handle, self._name, 0x121)

    @property
    def ao_max(self):
        """
        float: Specifies the maximum value you expect to generate. The
            value is in the units you specify with a units property. If
            you try to write a value larger than the maximum value, NI-
            DAQmx generates an error. NI-DAQmx might coerce this value
            to a smaller value if other task settings restrict the
            device from generating the desired maximum.
        """

        val = self._interpreter.get_chan_attribute_double(self._handle, self._name, 0x1186)
        return val

    @ao_max.setter
    def ao_max(self, val):
        self._interpreter.set_chan_attribute_double(self._handle, self._name, 0x1186, val)

    @ao_max.deleter
    def ao_max(self):
        self._interpreter.reset_chan_attribute(self._handle, self._name, 0x1186)

    @property
    def ao_mem_map_enable(self):
        """
        bool: Specifies for NI-DAQmx to map hardware registers to the
            memory space of the application, if possible. Normally, NI-
            DAQmx maps hardware registers to memory accessible only to
            the kernel. Mapping the registers to the memory space of the
            application increases performance. However, if the
            application accesses the memory space mapped to the
            registers, it can adversely affect the operation of the
            device and possibly result in a system crash.
        """

        val = self._interpreter.get_chan_attribute_bool(self._handle, self._name, 0x188f)
        return val

    @ao_mem_map_enable.setter
    def ao_mem_map_enable(self, val):
        self._interpreter.set_chan_attribute_bool(self._handle, self._name, 0x188f, val)

    @ao_mem_map_enable.deleter
    def ao_mem_map_enable(self):
        self._interpreter.reset_chan_attribute(self._handle, self._name, 0x188f)

    @property
    def ao_min(self):
        """
        float: Specifies the minimum value you expect to generate. The
            value is in the units you specify with a units property. If
            you try to write a value smaller than the minimum value, NI-
            DAQmx generates an error. NI-DAQmx might coerce this value
            to a larger value if other task settings restrict the device
            from generating the desired minimum.
        """

        val = self._interpreter.get_chan_attribute_double(self._handle, self._name, 0x1187)
        return val

    @ao_min.setter
    def ao_min(self, val):
        self._interpreter.set_chan_attribute_double(self._handle, self._name, 0x1187, val)

    @ao_min.deleter
    def ao_min(self):
        self._interpreter.reset_chan_attribute(self._handle, self._name, 0x1187)

    @property
    def ao_output_impedance(self):
        """
        float: Specifies in ohms the impedance of the analog output
            stage of the device.
        """

        val = self._interpreter.get_chan_attribute_double(self._handle, self._name, 0x1490)
        return val

    @ao_output_impedance.setter
    def ao_output_impedance(self, val):
        self._interpreter.set_chan_attribute_double(self._handle, self._name, 0x1490, val)

    @ao_output_impedance.deleter
    def ao_output_impedance(self):
        self._interpreter.reset_chan_attribute(self._handle, self._name, 0x1490)

    @property
    def ao_output_type(self):
        """
        :class:`nidaqmx.constants.UsageTypeAO`: Indicates whether the
            channel generates voltage,  current, or a waveform.
        """

        val = self._interpreter.get_chan_attribute_int32(self._handle, self._name, 0x1108)
        return UsageTypeAO(val)

    @property
    def ao_reglitch_enable(self):
        """
        bool: Specifies whether to enable reglitching.  The output of a
            DAC normally glitches whenever the DAC is updated with a new
            value. The amount of glitching differs from code to code and
            is generally largest at major code transitions.  Reglitching
            generates uniform glitch energy at each code transition and
            provides for more uniform glitches.  Uniform glitch energy
            makes it easier to filter out the noise introduced from
            glitching during spectrum analysis.
        """

        val = self._interpreter.get_chan_attribute_bool(self._handle, self._name, 0x133)
        return val

    @ao_reglitch_enable.setter
    def ao_reglitch_enable(self, val):
        self._interpreter.set_chan_attribute_bool(self._handle, self._name, 0x133, val)

    @ao_reglitch_enable.deleter
    def ao_reglitch_enable(self):
        self._interpreter.reset_chan_attribute(self._handle, self._name, 0x133)

    @property
    def ao_resolution(self):
        """
        float: Indicates the resolution of the digital-to-analog
            converter of the channel. This value is in the units you
            specify with **ao_resolution_units**.
        """

        val = self._interpreter.get_chan_attribute_double(self._handle, self._name, 0x182c)
        return val

    @property
    def ao_resolution_units(self):
        """
        :class:`nidaqmx.constants.ResolutionType`: Specifies the units
            of **ao_resolution**.
        """

        val = self._interpreter.get_chan_attribute_int32(self._handle, self._name, 0x182b)
        return ResolutionType(val)

    @ao_resolution_units.setter
    def ao_resolution_units(self, val):
        val = val.value
        self._interpreter.set_chan_attribute_int32(self._handle, self._name, 0x182b, val)

    @ao_resolution_units.deleter
    def ao_resolution_units(self):
        self._interpreter.reset_chan_attribute(self._handle, self._name, 0x182b)

    @property
    def ao_term_cfg(self):
        """
        :class:`nidaqmx.constants.TerminalConfiguration`: Specifies the
            terminal configuration of the channel.
        """

        val = self._interpreter.get_chan_attribute_int32(self._handle, self._name, 0x188e)
        return TerminalConfiguration(val)

    @ao_term_cfg.setter
    def ao_term_cfg(self, val):
        val = val.value
        self._interpreter.set_chan_attribute_int32(self._handle, self._name, 0x188e, val)

    @ao_term_cfg.deleter
    def ao_term_cfg(self):
        self._interpreter.reset_chan_attribute(self._handle, self._name, 0x188e)

    @property
    def ao_usb_xfer_req_count(self):
        """
        int: Specifies the maximum number of simultaneous USB transfers
            used to stream data. Modify this value to affect performance
            under different combinations of operating system and device.
        """

        val = self._interpreter.get_chan_attribute_uint32(self._handle, self._name, 0x3001)
        return val

    @ao_usb_xfer_req_count.setter
    def ao_usb_xfer_req_count(self, val):
        self._interpreter.set_chan_attribute_uint32(self._handle, self._name, 0x3001, val)

    @ao_usb_xfer_req_count.deleter
    def ao_usb_xfer_req_count(self):
        self._interpreter.reset_chan_attribute(self._handle, self._name, 0x3001)

    @property
    def ao_usb_xfer_req_size(self):
        """
        int: Specifies the maximum size of a USB transfer request in
            bytes. Modify this value to affect performance under
            different combinations of operating system and device.
        """

        val = self._interpreter.get_chan_attribute_uint32(self._handle, self._name, 0x2a8f)
        return val

    @ao_usb_xfer_req_size.setter
    def ao_usb_xfer_req_size(self, val):
        self._interpreter.set_chan_attribute_uint32(self._handle, self._name, 0x2a8f, val)

    @ao_usb_xfer_req_size.deleter
    def ao_usb_xfer_req_size(self):
        self._interpreter.reset_chan_attribute(self._handle, self._name, 0x2a8f)

    @property
    def ao_use_only_on_brd_mem(self):
        """
        bool: Specifies whether to write samples directly to the onboard
            memory of the device, bypassing the memory buffer.
            Generally, you cannot update onboard memory directly after
            you start the task. Onboard memory includes data FIFOs.
        """

        val = self._interpreter.get_chan_attribute_bool(self._handle, self._name, 0x183a)
        return val

    @ao_use_only_on_brd_mem.setter
    def ao_use_only_on_brd_mem(self, val):
        self._interpreter.set_chan_attribute_bool(self._handle, self._name, 0x183a, val)

    @ao_use_only_on_brd_mem.deleter
    def ao_use_only_on_brd_mem(self):
        self._interpreter.reset_chan_attribute(self._handle, self._name, 0x183a)

    @property
    def ao_voltage_current_limit(self):
        """
        float: Specifies the current limit, in amperes, for the voltage
            channel.
        """

        val = self._interpreter.get_chan_attribute_double(self._handle, self._name, 0x2a1d)
        return val

    @ao_voltage_current_limit.setter
    def ao_voltage_current_limit(self, val):
        self._interpreter.set_chan_attribute_double(self._handle, self._name, 0x2a1d, val)

    @ao_voltage_current_limit.deleter
    def ao_voltage_current_limit(self):
        self._interpreter.reset_chan_attribute(self._handle, self._name, 0x2a1d)

    @property
    def ao_voltage_units(self):
        """
        :class:`nidaqmx.constants.VoltageUnits`: Specifies in what units
            to generate voltage on the channel. Write data to the
            channel in the units you select.
        """

        val = self._interpreter.get_chan_attribute_int32(self._handle, self._name, 0x1184)
        return VoltageUnits(val)

    @ao_voltage_units.setter
    def ao_voltage_units(self, val):
        val = val.value
        self._interpreter.set_chan_attribute_int32(self._handle, self._name, 0x1184, val)

    @ao_voltage_units.deleter
    def ao_voltage_units(self):
        self._interpreter.reset_chan_attribute(self._handle, self._name, 0x1184)




# Do not edit this file; it was automatically generated.

import nidaqmx.task.channels  # circular import: Channel._factory uses derived classes
from nidaqmx.system.physical_channel import _PhysicalChannelAlternateConstructor
from nidaqmx.utils import flatten_channel_string, unflatten_channel_string
from nidaqmx.constants import (
    ChannelType, SyncUnlockBehavior, _Save)


class Channel:
    """
    Represents virtual channel or a list of virtual channels.
    """
    __slots__ = ['_handle', '_name', '_interpreter', '__weakref__']

    def __init__(self, task_handle, virtual_or_physical_name, interpreter):
        """
        Args:
            task_handle (TaskHandle): Specifies the handle of the task that
                this channel is associated with.
            virtual_or_physical_name (str): Specifies the flattened virtual or
                physical name of a channel.
            interpreter (BaseInterpreter): Specifies the interpreter instance.
        """
        self._handle = task_handle
        self._name = virtual_or_physical_name
        self._interpreter = interpreter

    def __add__(self, other):
        if not isinstance(other, self.__class__):
            raise NotImplementedError(
                'Cannot concatenate objects of type {} and {}'
                .format(self.__class__, other.__class__))

        if self._handle != other._handle:
            raise NotImplementedError(
                'Cannot concatenate Channel objects from different tasks.')

        name = flatten_channel_string([self.name, other.name])
        return Channel._factory(self._handle, name, self._interpreter)

    def __contains__(self, item):
        channel_names = self.channel_names

        if isinstance(item, str):
            items = unflatten_channel_string(item)
        elif isinstance(item, Channel):
            items = item.channel_names

        return all([item in channel_names for item in items])

    def __eq__(self, other):
        if isinstance(other, self.__class__):
            return (self._handle == other._handle and
                    set(self.channel_names) == set(other.channel_names))
        return False

    def __hash__(self):
        return self._interpreter.hash_task_handle(self._handle) ^ hash(frozenset(self.channel_names))

    def __iadd__(self, other):
        return self.__add__(other)

    def __iter__(self):
        for channel_name in self.channel_names:
            yield Channel._factory(self._handle, channel_name, self._interpreter)

    def __len__(self):
        return len(self.channel_names)

    def __ne__(self, other):
        return not self.__eq__(other)

    def __reversed__(self):
        channel_names = self.channel_names
        channel_names.reverse()

        for channel_name in channel_names:
            yield Channel._factory(self._handle, channel_name, self._interpreter)

    def __repr__(self):
        return f'Channel(name={self.name})'

    @staticmethod
    def _factory(task_handle, virtual_or_physical_name, interpreter):
        """
        Implements the factory pattern for nidaqmx channels.

        Args:
            task_handle (TaskHandle): Specifies the handle of the task that
                this channel is associated with.
            virtual_or_physical_name (str): Specifies the flattened virtual
                or physical name of a channel.
            interpreter (BaseInterpreter): Specifies the interpreter instance.
        Returns:
            nidaqmx.task.channels.Channel:

            Indicates an object that represents the specified channel.
        """
        chan_type = interpreter.get_chan_attribute_int32(task_handle, virtual_or_physical_name, 0x187f)

        channel_type = ChannelType(chan_type)

        if channel_type == ChannelType.ANALOG_INPUT:
            return nidaqmx.task.channels.AIChannel(
                task_handle, virtual_or_physical_name, interpreter)
        elif channel_type == ChannelType.ANALOG_OUTPUT:
            return nidaqmx.task.channels.AOChannel(
                task_handle, virtual_or_physical_name, interpreter)
        elif channel_type == ChannelType.COUNTER_INPUT:
            return nidaqmx.task.channels.CIChannel(
                task_handle, virtual_or_physical_name, interpreter)
        elif channel_type == ChannelType.COUNTER_OUTPUT:
            return nidaqmx.task.channels.COChannel(
                task_handle, virtual_or_physical_name, interpreter)
        elif channel_type == ChannelType.DIGITAL_INPUT:
            return nidaqmx.task.channels.DIChannel(
                task_handle, virtual_or_physical_name, interpreter)
        elif channel_type == ChannelType.DIGITAL_OUTPUT:
            return nidaqmx.task.channels.DOChannel(
                task_handle, virtual_or_physical_name, interpreter)

    @property
    def name(self):
        """
        str: Specifies the name of the virtual channel this object
            represents.
        """
        if self._name:
            return self._name
        else:
            return self._all_channels_name

    @property
    def channel_names(self):
        """
        List[str]: Specifies the unflattened list of the virtual channels.
        """
        if self._name:
            return unflatten_channel_string(self._name)
        else:
            return unflatten_channel_string(self._all_channels_name)

    @property
    def _all_channels_name(self):
        """
        str: Specifies the flattened names of all the virtual channels in
            the task.
        """
        val = self._interpreter.get_task_attribute_string(self._handle, 0x1273)
        return val

    @property
    def chan_type(self):
        """
        :class:`nidaqmx.constants.ChannelType`: Indicates the type of
            the virtual channel.
        """

        val = self._interpreter.get_chan_attribute_int32(self._handle, self._name, 0x187f)
        return ChannelType(val)

    @property
    def description(self):
        """
        str: Specifies a user-defined description for the channel.
        """

        val = self._interpreter.get_chan_attribute_string(self._handle, self._name, 0x1926)
        return val

    @description.setter
    def description(self, val):
        self._interpreter.set_chan_attribute_string(self._handle, self._name, 0x1926, val)

    @description.deleter
    def description(self):
        self._interpreter.reset_chan_attribute(self._handle, self._name, 0x1926)

    @property
    def is_global(self):
        """
        bool: Indicates whether the channel is a global channel.
        """

        val = self._interpreter.get_chan_attribute_bool(self._handle, self._name, 0x2304)
        return val

    @property
    def physical_channel(self):
        """
        :class:`nidaqmx.system.physical_channel.PhysicalChannel`:
            Specifies the name of the physical channel upon which this
            virtual channel is based.
        """

        val = self._interpreter.get_chan_attribute_string(self._handle, self._name, 0x18f5)
        return _PhysicalChannelAlternateConstructor(val, self._interpreter)

    @physical_channel.setter
    def physical_channel(self, val):
        val = val.name
        self._interpreter.set_chan_attribute_string(self._handle, self._name, 0x18f5, val)

    @property
    def sync_unlock_behavior(self):
        """
        :class:`nidaqmx.constants.SyncUnlockBehavior`: Specifies the
            action to take if the target loses its synchronization to
            the grand master.
        """

        val = self._interpreter.get_chan_attribute_int32(self._handle, self._name, 0x313c)
        return SyncUnlockBehavior(val)

    @sync_unlock_behavior.setter
    def sync_unlock_behavior(self, val):
        val = val.value
        self._interpreter.set_chan_attribute_int32(self._handle, self._name, 0x313c, val)

    @sync_unlock_behavior.deleter
    def sync_unlock_behavior(self):
        self._interpreter.reset_chan_attribute(self._handle, self._name, 0x313c)

    def save(self, save_as="", author="", overwrite_existing_channel=False,
             allow_interactive_editing=True, allow_interactive_deletion=True):
        """
        Saves this local or global channel to MAX as a global channel.

        Args:
            save_as (Optional[str]): Is the name to save the task,
                global channel, or custom scale as. If you do not
                specify a value for this input, NI-DAQmx uses the name
                currently assigned to the task, global channel, or
                custom scale.
            author (Optional[str]): Is a name to store with the task,
                global channel, or custom scale.
            overwrite_existing_channel (Optional[bool]): Specifies whether to
                overwrite a global channel of the same name if one is already
                saved in MAX. If this input is False and a global channel of
                the same name is already saved in MAX, this function returns
                an error.
            allow_interactive_editing (Optional[bool]): Specifies whether to
                allow the task, global channel, or custom scale to be edited
                in the DAQ Assistant. If allow_interactive_editing is True,
                the DAQ Assistant must support all task or global channel
                settings.
            allow_interactive_deletion (Optional[bool]): Specifies whether
                to allow the task, global channel, or custom scale to be
                deleted through MAX.
        """
        options = 0
        if overwrite_existing_channel:
            options |= _Save.OVERWRITE.value
        if allow_interactive_editing:
            options |= _Save.ALLOW_INTERACTIVE_EDITING.value
        if allow_interactive_deletion:
            options |= _Save.ALLOW_INTERACTIVE_DELETION.value

        self._interpreter.save_global_chan(self._handle, self._name, save_as, author, options)



from collections.abc import Sequence

from nidaqmx.errors import DaqError
from nidaqmx.error_codes import DAQmxErrors
from nidaqmx.system.storage.persisted_scale import PersistedScale, _PersistedScaleAlternateConstructor
from nidaqmx.utils import unflatten_channel_string

class PersistedScaleCollection(Sequence):
    """
    Contains the collection of custom scales on a DAQmx system.
    
    This class defines methods that implements a container object.
    """
    def __init__(self, interpreter):
        """
        Do not construct this object directly; instead, call nidaqmx.system.System.local().scales.
        """
        self._interpreter = interpreter
    
    def __contains__(self, item):
        scale_names = self.scale_names

        if isinstance(item, str):
            items = unflatten_channel_string(item)
            return all([i in scale_names for i in items])
        elif isinstance(item, PersistedScale):
            return item._name in scale_names

    def __eq__(self, other):
        if isinstance(other, self.__class__):
            return True
        return False

    def __getitem__(self, index):
        """
        Indexes a subset of custom scales on this collection.

        Args:
            index: The value of the index. The following index types
                are supported:
                - str: Name of the custom scale. You also can specify
                    a string that contains a list or range of names to
                    this input. If you have a list of names, use the
                    DAQmx Flatten Channel String function to convert
                    the list to a string.
                - int: Index/position of the custom scale in the
                    collection.
                - slice: Range of the indexes/positions of custom scales
                    in the collection.
        Returns:
            List[nidaqmx.system.storage.persisted_scale.PersistedScale]:
            
            Indicates the subset of custom scales indexed.
        """
        if isinstance(index, int):
            return _PersistedScaleAlternateConstructor(self.scale_names[index], self._interpreter)
        elif isinstance(index, slice):
            return [_PersistedScaleAlternateConstructor(name, self._interpreter) for name in
                    self.scale_names[index]]
        elif isinstance(index, str):
            names = unflatten_channel_string(index)
            if len(names) == 1:
                return _PersistedScaleAlternateConstructor(names[0], self._interpreter)
            return [_PersistedScaleAlternateConstructor(name, self._interpreter) for name in names]
        else:
            raise DaqError(
                'Invalid index type "{}" used to access collection.'
                .format(type(index)), DAQmxErrors.UNKNOWN)

    def __iter__(self):
        for scale_name in self.scale_names:
            yield _PersistedScaleAlternateConstructor(scale_name, self._interpreter)

    def __len__(self):
        return len(self.scale_names)

    def __ne__(self, other):
        return not self.__eq__(other)

    def __reversed__(self):
        scale_names = self.scale_names
        scale_names.reverse()

        for scale_name in scale_names:
            yield _PersistedScaleAlternateConstructor(scale_name, self._interpreter)

    @property
    def scale_names(self):
        """
        List[str]: Indicates the names of all the custom scales on this
            collection.
        """
        val = self._interpreter.get_system_info_attribute_string(0x1266)
        return unflatten_channel_string(val)
